#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/talk.sh - configuration generator script for talk service
#
# Copyright (c) 2002-2012 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2005-05-22  jed
# Last Update:  $Id: talk.sh 31572 2012-09-02 09:13:49Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib
. /var/install/include/eislib

# exec 2>/tmp/talk-trace-$$.log
# set -x

#testroot=/soft/jedtalk                                          # only for testing
 testroot=""

pgmname=`basename $0`

### set file names ###
talkfile=$testroot/etc/config.d/talk
generate_talkconf=$testroot/etc/xinetd.d/talk
generate_ytalkconf=$testroot/etc/ytalkrc
utmp_file=$testroot/var/run/utmp
debug_logfile=/var/log/talkd.log
debug_packetfile=/var/log/talkd.packets

### other parameters ###
talk_version="v1.1.1-1"

### load configuration ###
. $talkfile
chmod 600 $talkfile

#----------------------------------------------------------------------------------------
# creating xinetd talk configuration
# $1 - no-init of xinet after reconfigure
#----------------------------------------------------------------------------------------
create_xinet_talk ()
{
    noinit=$1

    mecho "creating talk configuration file ..."

    trusted_networks=''
    idx=1
    while [ $idx -le $TALK_TRUSTED_NETS_N ]
    do
        eval trusted_net='$TALK_TRUSTED_NETS_'$idx

        if [ "$trusted_networks" = "" ]
        then
            trusted_networks="$trusted_net"
        else
            trusted_networks="$trusted_networks $trusted_net"
        fi

        idx=`expr $idx + 1`
    done

    {
        echo "#------------------------------------------------------------------"
        echo "# $generate_talkconf file generated by $pgmname $talk_version"
        echo "#"
        echo "# Do not edit this file, edit $talkfile"
        echo "# Creation date: `date`"
        echo "#------------------------------------------------------------------"
        echo
        echo "service talk"
        echo "{"
        echo "  socket_type = dgram"
        echo "  protocol    = udp"
        echo "  wait   = yes"
        echo "  user   = nobody"
        echo "  group  = tty"
        echo "  server = /usr/sbin/talkd"

        if [ "$TALK_DO_DEBUG" = "yes" ]
        then
            # activate debug logging
            echo "  server_args = -d -p"
        fi

        if [ $TALK_TRUSTED_NETS_N -gt 0 ]
        then
            echo "  only_from = $trusted_networks"
        fi
    

        if [ "$START_TALK" = "yes" ]
        then
            echo "  disable = no"
        else
            echo "  disable = yes"
        fi

        echo "}"
        echo
        echo "service ntalk"
        echo "{"
        echo "  socket_type = dgram"
        echo "  protocol    = udp"
        echo "  wait   = yes"
        echo "  user   = nobody"
        echo "  group  = tty"
        echo "  server = /usr/sbin/ntalkd"

        if [ "$TALK_DO_DEBUG" = "yes" ]
        then
            # activate debug logging
            echo "  server_args = -d -p"
        fi

        if [ $TALK_TRUSTED_NETS_N -gt 0 ]
        then
            echo "  only_from = $trusted_networks"
        fi

        if [ "$START_TALK" = "yes" ]
        then
            echo "  disable = no"
        else
            echo "  disable = yes"
        fi

        echo "}"
    } >$generate_talkconf

    if [ "$noinit" != "noinit" ]
    then
        if [ "$START_TALK" = "yes" ]
        then
            killall -1 xinetd 2>/dev/null

            if [ $? != 0 ]
            then
                mecho -error "talk daemon needs xinetd which is not running."
            fi
        else
            killall -1 inetd 2>/dev/null
        fi
    fi
}

#----------------------------------------------------------------------------------------
# creating ytalk configuration
#----------------------------------------------------------------------------------------
create_ytalk ()
{
    mecho "creating ytalk configuration file ..."

    {
        echo "#------------------------------------------------------------------"
        echo "# $generate_ytalkconf file generated by $pgmname $talk_version"
        echo "#"
        echo "# Do not edit this file, edit $talkfile"
        echo "# Creation date: `date`"
        echo "#------------------------------------------------------------------"
        echo

        if [ "$TALK_SCROLLING" = "yes" ]
        then
            echo "turn scrolling on"
        else
            echo "turn scrolling off"
        fi

        if [ "$TALK_WORDWRAP" = "yes" ]
        then
            echo "turn word-wrap on"
        else
            echo "turn word-wrap off"
        fi

        if [ "$TALK_AUTO_INVITE" = "yes" ]
        then
            echo "turn auto-invite on"
        else
            echo "turn auto-invite off"
        fi

        if [ "$TALK_RERING" = "yes" ]
        then
            echo "turn rering on"
        else
            echo "turn rering off"
        fi

        if [ "$TALK_RERING_PROMPT" = "yes" ]
        then
            echo "turn prompt-rering on"
        else
            echo "turn prompt-rering off"
        fi
    } >$generate_ytalkconf
}

#----------------------------------------------------------------------------------------
# create debug log and packet file
#----------------------------------------------------------------------------------------
create_debug_log ()
{
    if [ "$TALK_DO_DEBUG" = "yes" ]
    then
        # activate debug logging
        mecho "enable debug logging ..."

        if [ ! -f $debug_logfile ]
        then
            # create log file
            > $debug_logfile
            chown nobody $debug_logfile
        fi

        if [ ! -f $debug_packetfile ]
        then
            # create packet file
            > $debug_packetfile
            chown nobody $debug_packetfile
        fi
    else
        # no debug mode
      # mecho "disable debug logging ..."

        if [ -f $debug_logfile ]
        then
            # remove log file
            rm -p $debug_logfile
        fi

        if [ -f $debug_packetfile ]
        then
            # remove packet file
            rm -p $debug_packetfile
        fi
    fi
}

#----------------------------------------------------------------------------------------
# check permissioning
#----------------------------------------------------------------------------------------
check_permissions ()
{
    mecho "checking permissions ..."

    if [ -f $utmp_file ]
    then
        # -rw-r--r--
        rights_mask=`ls -l $utmp_file|cut -d' ' -f1`
        # r
        read_access=`echo "$rights_mask"|cut -c8`

        if [ "$read_access" != "r" ]
        then
            mecho -error "Invalid access rights of file '$utmp_file' ($rights_mask)."
            mecho "Setting read access for everyone (chmod o+r $utmp_file) ..."

            chmod o+r $utmp_file
        fi
    else
        mecho -error "file '$utmp_file' doesn't exist!"
    fi
}

#========================================================================================
# Main
#========================================================================================

case "$1"
in
    rewritexinetd)
        # rewrite xinetd talk configuration
        create_xinet_talk
        ;;
    *)
        # generate all configuration files
        mecho
        mecho "version: $talk_version"

        create_xinet_talk
        create_ytalk
        create_debug_log

        check_permissions

        mecho "finished."
esac

#========================================================================================
# End
#========================================================================================
exit 0
